@@ -14,7 +14,7 @@
    You should have received a copy of the GNU General Public License
    along with Redshift.  If not, see <http://www.gnu.org/licenses/>.
 
-   Copyright (c) 2013-2014  Jon Lund Steffensen <jonlst@gmail.com>
+   Copyright (c) 2013-2017  Jon Lund Steffensen <jonlst@gmail.com>
 */
 
 #ifndef REDSHIFT_GAMMA_DUMMY_H
@@ -22,17 +22,6 @@
 
 #include "redshift.h"
 
-
-int gamma_dummy_init(void *state);
-int gamma_dummy_start(void *state);
-void gamma_dummy_free(void *state);
-
-void gamma_dummy_print_help(FILE *f);
-int gamma_dummy_set_option(void *state, const char *key, const char *value);
-
-void gamma_dummy_restore(void *state);
-int gamma_dummy_set_temperature(void *state,
-				const color_setting_t *setting);
-
+extern const gamma_method_t dummy_gamma_method;
 
 #endif /* ! REDSHIFT_GAMMA_DUMMY_H */
