@@ -14,27 +14,14 @@
    You should have received a copy of the GNU General Public License
    along with Redshift.  If not, see <http://www.gnu.org/licenses/>.
 
-   Copyright (c) 2014  Jon Lund Steffense <jonlst@gmail.com>
+   Copyright (c) 2014-2017  Jon Lund Steffense <jonlst@gmail.com>
 */
 
 #ifndef REDSHIFT_LOCATION_CORELOCATION_H
 #define REDSHIFT_LOCATION_CORELOCATION_H
 
-#include <stdio.h>
-
 #include "redshift.h"
 
-
-int location_corelocation_init(void *state);
-int location_corelocation_start(void *state);
-void location_corelocation_free(void *state);
-
-void location_corelocation_print_help(FILE *f);
-int location_corelocation_set_option(void *state,
-				     const char *key, const char *value);
-
-int location_corelocation_get_location(void *state,
-				       location_t *location);
-
+extern const location_provider_t corelocation_location_provider;
 
 #endif /* ! REDSHIFT_LOCATION_CORELOCATION_H */
